// ==UserScript==
// @name          gdocs-unity-integration
// @include       https://docs.google.com/document/*
// @include       https://docs.google.com/present/*
// @include       https://docs.google.com/spreadsheet/*
// @include       https://docs.google.com/drawings/*
// @include       https://docs.google.com/*/document/*
// @include       https://docs.google.com/*/present/*
// @include       https://docs.google.com/*/spreadsheet/*
// @include       https://docs.google.com/*/drawings/*
// @version       0.9
// @author        WebApps Team
// @require       utils.js
// @require       google-common.js
// ==/UserScript==

var menu = {};

function getOffsetTop(node) {
    return node.offsetTop + (node.offsetParent ? getOffsetTop(node.offsetParent) : 0);
}

function getOffsetLeft(node) {
    return node.offsetLeft + (node.offsetParent ? getOffsetLeft(node.offsetParent) : 0);
}

function registerSubMenu(id, node) {
    var i, k, parent = document.getElementById(id).textContent;

    if (menu[id].childs.length) {
        return;
    }

    var res = document.evaluate('div[@class="goog-menuitem"]/div[@class="goog-menuitem-content"]',
                                node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    function activateMenu() {
        click(this);
    }
    for (i = 0; i < res.snapshotLength; i++) {
        var item = res.snapshotItem(i);
        var textContent = '';
        for (k = 0; k < item.childNodes.length; k++) {
            if (item.childNodes[k].nodeName === '#text' || item.childNodes[k].className === 'goog-menuitem-mnemonic-hint'
                    || item.childNodes[k].nodeName === 'STRONG') {
                textContent += item.childNodes[k].textContent;
            }
        }

        if (textContent.length) {
            var path = '/' + parent + '/' + textContent;
            console.log(path);
            Unity.addAction(path, activateMenu.bind(item));
        }
    }
}

function registerMenu() {
    function matchMenuAndSubMenu() {
        var x, y, k;
        x = getOffsetLeft(this);
        y = getOffsetTop(this);

        if (!y) {
            return;
        }

        for (k in menu) {
            if (menu.hasOwnProperty(k) && menu[k].x === x && Math.abs(menu[k].y - y) < 40) {
                registerSubMenu(k, this);
                this.removeEventListener('DOMSubtreeModified', this._callback, false);
            }
        }
    }
    var i, elements = document.getElementsByClassName('goog-menu');

    if (!elements) {
        return;
    }

    for (i = 0; i < elements.length; i++) {
        var node = elements[i];
        if (!node._callback) {
            node._callback = matchMenuAndSubMenu.bind(node);
            node.addEventListener('DOMSubtreeModified', node._callback, false);
        }

    }
}

function doIntegration() {
    var i, node, elements = document.getElementsByClassName('menu-button goog-control goog-inline-block');

    for (i = 0; i < elements.length; i++) {
        node = elements[i];
        var x = getOffsetLeft(node);
        var y = getOffsetTop(node);
        menu[node.id] = { x: x, y: y, childs: [] };
    }

    var timeout = 100;

    setTimeout(function repeat() {
        registerMenu();
        setTimeout(repeat, timeout);
        if (timeout < 10000) {
            timeout *= 2;
        }
    }, timeout);

    if (document.getElementById('docs-title')) {
        var uri = window.location;
        uri = uri.hostname + uri.pathname + uri.search;
        linkVisited(uri, document.getElementById('docs-title').textContent);
    }
    addFavoritesInLauncher();

    doMainMenuIntegration(document);
}

Unity.init({ name: _("Google Docs"),
             iconUrl: 'http://www.google.com/images/icons/product/docs-128.png',
             onInit: doIntegration });
